import { Vector3 } from 'common/math.js';

// offset source radius {x-power y-power} blacklist
export function knockbackDome(offset, source, radius, power, blacklist){
	source.dimension.getEntities({
		location:yLessOffset(source, offset),
		maxDistance:radius,
		excludeFamilies:blacklist
	}).forEach(entity =>{
		if(entity.typeId == 'minecraft:player'){
			if(source.name != entity.name){
				doKnockback(source, entity, power);
			}
		}
		else{
			doKnockback(source, entity, power);
		}
	});
}

// offset source radius power blacklist
export function absorbDome(offset, source, radius, power, blacklist){
	source.dimension.getEntities({
		location:yLessOffset(source, offset),
		maxDistance:radius,
		excludeFamilies:blacklist
	}).forEach(entity =>{
		if(entity.typeId == 'minecraft:player'){
			if(source.name != entity.name){
				doAbsorb(source, entity, power);
			}
		}
		else{
			doAbsorb(source, entity, power);
		}
	});
}

// creates a knockback force based on the location
// of the source and target's coordinates. By adding a
// boolean at the end of the function, you can create
// a pullback force instead.
// source target {x-power y-power}
export function doKnockback(source, target, power){
	let angle = getAngle(source.location, target.location);
	power.y == undefined ? power.y = 0 : null;
	try{
		if(target.typeId == 'minecraft:player'){
			if(target.getGameMode() != 'creative'){
				target.applyKnockback(angle.x, angle.z, power.x / 1.5, power.y / 6);
			}
		}
		else{
			target.applyKnockback(angle.x, angle.z, power.x / 1.5, power.y / 6);
		}
	}
	catch{}
}

// creates a knockback force based on the location
// of the source and target's coordinates. By adding a
// boolean at the end of the function, you can create
// a pullback force instead.
// source target {x-power y-power}
export function doAbsorb(source, target, power){
	let angle = getAngle(source.location, target.location);
	try{
		if(target.typeId == 'minecraft:player'){
			if(target.getGameMode() != 'creative'){
				target.applyKnockback(angle.x, angle.z, -power / 1.5, (-power * angle.y) / 6);
			}
		}
		else{
			target.applyKnockback(angle.x, angle.z, -power / 1.5, (-power * angle.y) / 6);
		}
	}
	catch{}
}

// it returns an angle vector
// from-coords to-coords
export function getAngle(from, to){
  let stepsX = to.x - from.x;
  let stepsY = to.y - from.y;
  let stepsZ = to.z - from.z;
  return { x:stepsX, y:stepsY, z:stepsZ };
}

// returns a vector based on the source's view direction
export function carrotOffset(source, offset){
	var view = source.getViewDirection();
	let normalized = Vector3.Normalize(view);
	return Vector3.Add(source.getHeadLocation(), Vector3.Multiply(normalized, {x:offset, y:offset, z:offset}));
}

// returns a vector based on the source's view direction without the y axis
export function yLessOffset(source, offset){
	var view = source.getViewDirection();
	let { x, z } = Vector3.Normalize({x: view.x, y:0, z:view.z});
	return Vector3.Add(source.location, {x:x * offset, y:0, z:z * offset});
}