import { damageArea, wireParticle, jumpMechanic } from 'true_carnage/functions.js';
import { doAbsorb, absorbDome, knockbackDome, yLessOffset, carrotOffset, getAngle } from 'essential/functions.js';
import { Vector3 } from 'common/math.js';

export const events = {
	"raigen:add_id": function(e){
		e.setDynamicProperty("raigen:entity_id", Math.random());
	},
	"raigen:broot_attack": function(e){
		let id = e.getComponent("skin_id").value;
		switch(id){
			case 1:
				try{
					let target = e.target;
					if(target == undefined) return;
					if(target.isOnGround){
						target.runCommand("effect @s slowness 5 255 true");
						target.dimension.spawnEntity("raigen:broot_roots", target.location);
					}
				}
				catch{}
				break;
			case 2:
			case 3:
				damageArea(1, e, 2.5, 6, ["ent", "inanimate"]);
				break;
			default:
				break;
		}
	},
	"raigen:vesparc_attack": function(e){
		shockAttack(e);
	},
	"raigen:vesparc_attack_lightning": function(e){
		shockAttack(e);
		if(e.target == undefined) return;
		e.dimension.spawnEntity("minecraft:lightning_bolt", e.target.location);
	},
	"raigen:frayle_attack": function(e){
		let id = e.getComponent("skin_id").value;
		switch(id){
			case 1:
			case 2:
				damageArea(1, e, 1.5, 9, ["frayle", "inanimate"]);
				break;
			case 3:
				damageArea(1, e, 1.5, 18, ["frayle", "inanimate"]);
				break;
			case 4:
			case 5:
				damageArea(1, e, 0.8, 13, ["frayle", "inanimate"]);
				break;
			case 6:
				damageArea(1, e, 0.8, 26, ["frayle", "inanimate"]);
				break;
			case 7:
			case 8:
				damageArea(1, e, 1.9, 18, ["frayle", "inanimate"]);
				damageArea(2.25, e, 1.9, 18, ["frayle", "inanimate"]);
				e.applyImpulse({x:e.getViewDirection().x * 2, y:0, z:e.getViewDirection().z * 2});
				break;
			case 9:
				damageArea(1, e, 1.9, 36, ["frayle", "inanimate"]);
				damageArea(2.25, e, 1.9, 36, ["frayle", "inanimate"]);
				e.applyImpulse({x:e.getViewDirection().x * 2, y:0, z:e.getViewDirection().z * 2});
				break;
			default:
				break;
		}
	},
	"raigen:lava_cube_jump": function(e){
		if(!e.isOnGround) return;
		jumpMechanic(e, 0.6, true, "mob.slime.big");
	},
	"raigen:jiangshi_jump": function(e){
		if(!e.isOnGround) return;
		jumpMechanic(e, 0.35, false, "", 0.35);
	},
	"raigen:black_grime_jump": function(e){
		if(!e.isOnGround) return;
		jumpMechanic(e, 0.7, true, "mob.slime.big", 0.5);
	},
	"raigen:globble_gob_jump": function(e){
		if(!e.isOnGround) return;
		jumpMechanic(e, 0.35, false, "mob.slime.big", 0.35);
	},
	"raigen:whisper_hp_attack": function(e){
		e.target?.applyDamage(e.target?.getComponent('health').effectiveMax / 10);
	},
	"raigen:rupture_bug_burst": function(e){
		damageArea(0, e, 4, 30, ["arthropod", "inanimate"]);
		knockbackDome(0, e, 4, {x:5, y:5}, ["arthropod"]);
	},
	"raigen:rampage_attack": function(e){
		let id = e.getComponent("skin_id").value;
		let va = e.getComponent("variant").value + 1;
		switch(id){
			case 1:
				damageArea(1, e, 1.5, 32 * va, ["terraflare", "inanimate"]);
				knockbackDome(1, e, 2.25, {x:20 * va, y:0}, ["terraflare"]);
				break;
			case 2:
				damageArea(0.5, e, 4, 16 * va, ["terraflare", "inanimate"]);
				knockbackDome(0.5, e, 4, {x:-1 * va, y:3 * va}, ["terraflare"]);
				break;
			case 3:
				damageArea(1, e, 4, 24 * va, ["terraflare", "inanimate"]);
				break;
			default:
				break;
		}
	},
	"raigen:tall_mimic_attack": function(e){
		let id = e.getComponent("skin_id").value;
		switch(id){
			case 1:
			case 2:
				damageArea(0.5, e, 2.5, 12, ["tall_mimic", "inanimate"]);
				knockbackDome(0.5, e, 2.5, {x:3, y:1.5}, ["tall_mimic"]);
			case 3:
				damageArea(1.25, e, 1.25, 30, ["tall_mimic", "inanimate"]);
			case 4:
			case 5:
				damageArea(0.5, e, 3, 6, ["tall_mimic", "inanimate"]);
			default:
				break;
		}
	},
	"raigen:shivers_freeze_area": function(e){
		e.dimension.getEntities({
			location:yLessOffset(e, 2.75),
			maxDistance:2.5,
			excludeFamilies:["undead"]
		}).forEach(entity =>{
			if(entity.typeId == 'minecraft:player'){
				if(entity.getGameMode() != 'creative'){
					entity.applyDamage(2, {cause:"freezing"});
					entity.addEffect("slowness", 200, {amplifier:3});
				}
			}
			else{
				entity.applyDamage(2, {cause:"freezing"});
				entity.addEffect("slowness", 200, {amplifier:3});
			}
		});
	},
	"raigen:bloodcurdle_detach": function(e){
		let variant = e.getComponent("variant").value;
		let parts = {
			"right_leg": 0,
			"left_leg": 0,
			"right_arm": 0.8,
			"left_arm": 0.8,
			"body": 0.8,
			"head": 1.6
		};
		Object.keys(parts).forEach(part =>{
			let body = e.dimension.spawnEntity(`raigen:bloodcurdle_body_part<raigen:event_${part}_${variant}>`, {x:e.location.x, y:e.location.y + parts[part], z:e.location.z});
			body.applyImpulse({x:-0.5 + Math.random() * 1, y:Math.random() * 0.5, z:-0.5 + Math.random() * 1});
			body.setRotation({x:0, y:Math.random() * 360});
		});
		e.triggerEvent("raigen:event_despawn");
	},
	"raigen:globble_gob_pull": function(e){
		absorbDome(0, e, 10, 0.5, ["slime", "inanimate"]);
	},
	"raigen:mootaur_attack": function(e){
		let id = e.getComponent("skin_id").value;
		switch(id){
			case 1:
			case 2:
				damageArea(1, e, 2.5, 12, ["cow", "inanimate"]);
				knockbackDome(1, e, 2.5, {x:6, y:0}, ["cow", "inanimate"]);
				break;
			case 3:
				damageArea(1, e, 2.5, 16, ["cow", "inanimate"]);
				knockbackDome(1, e, 2.5, {x:2, y:6}, ["cow", "inanimate"]);
				break;
			case 4:
				damageArea(1, e, 2.5, 20, ["cow", "inanimate"]);
				break;
			case 5:
				damageArea(1, e, 2, 28, ["cow", "inanimate"]);
				damageArea(1.5, e, 2, 28, ["cow", "inanimate"]);
				break;
			default:
				break;
		}
	},
	"raigen:branck_attack": function(e){
		if(e.target == undefined) return;
		e.teleport(carrotOffset(e.target, 0.75));
		e.setRotation({x:e.target.getRotation().x * -1, y:e.target.getRotation().y - 180});
		e.target.applyDamage(1);
	},
	"raigen:cavalrot_attack": function(e){
		e.applyImpulse({x:e.getViewDirection().x, y:0.7 + Math.random() * 0.2, z:e.getViewDirection().z});
	},
	"raigen:razor_attack": function(e){
		if(e.target != undefined){
			e.applyImpulse(Vector3.Multiply(getAngle(e.location, e.target.location), {x:0.45, y:0.45, z:0.45}));
			e.addEffect("invisibility", 10, {amplifier:1, showParticles:false});
			e.addEffect("resistance", 10, {amplifier:4, showParticles:false});
			e.target.applyDamage(16);
			e.dimension.spawnParticle("raigen:razor_slash", e.target.location);
			e.dimension.playSound("true_carnage.razor.attack", e.target.location);
		}
	},
	"raigen:skull_train_attack": function(e){
		if(e.target != undefined){
			e.addEffect("speed", 60, {amplifier:2, showParticles:false});
			e.dimension.playSound("true_carnage.skull_train.attack", e.location);
			e.dimension.playSound("true_carnage.skull_train.growl", e.location);
		}
	},
	"raigen:naverin_sprites": function(e){
		let sprite = e.dimension.spawnEntity("raigen:naverin_sprite", e.location);
		sprite.setDynamicProperty("raigen:entity_id", e.getDynamicProperty("raigen:entity_id"));
		sprite.applyImpulse({x:-0.5 + Math.random(), y:-0.5 + Math.random(), z:-0.5 + Math.random()});
	},
	"raigen:naverin_defense": function(e){
		knockbackDome(0, e, 3, {x:2, y:2}, ["naverin", "inanimate"]);
		e.dimension.spawnParticle("raigen:naverin_defense", {x:e.location.x, y:e.location.y + 1, z:e.location.z});
		e.dimension.playSound("true_carnage.runimon.shockwave", e.location);
	},
	"raigen:agtarae_attack": function(e){
		damageArea(1.5, e, 3, 42, ["fish", "inanimate"])
	},
	"raigen:zlato_spike": function(e){
		e.dimension.getEntities({
			location:e.location,
			maxDistance:10,
			excludeFamilies:["augen", "inanimate"]
		}).forEach(i=>{
			if(i.isOnGround){
				i.dimension.spawnEntity("raigen:zlato_spike", i.location);
			}
		})
	},
	"raigen:mortemeer_death": function(e){
		let c = 5 + Math.floor(Math.random() * 15);
		for(let i = 0; i < c; i++){
			let maggot = e.dimension.spawnEntity("raigen:maggot", Vector3.Add(e.location, {x:-0.5 + Math.random(), y:Math.random(), z:-0.5 + Math.random()}));
			maggot.applyKnockback(0, 0, 0, (0.5 + Math.random() * 0.35));
			maggot.setRotation(e.getRotation());
		}
	},
	"raigen:shadow_teleport": function(e){
		if(e.target == undefined) return;
		e.teleport(e.target.location);
	},
	"raigen:noir_attack": function(e){
		if(e.target == undefined) return;
		wireParticle(e.location, e.target.location, 0.1, e, "raigen:noir_laser");
		e.target?.applyDamage(5);
		e.dimension.playSound("mob.guardian.death", e.location);
		e.target?.typeId == "minecraf:player" ? e.target.runCommand("camera @s fade time 0 1 1 color 0 0 0") : null;
	},
	"raigen:dune_maiden_knock": function(e){
		try{
			e.applyImpulse({x:0, y:1, z:0});
		}
		catch{
			e.applyKnockback(0, 0, 0, 1);
		}
	},
	"raigen:hailrog_attack": function(e){
		let posX = Math.floor(-8 + Math.random() * 16);
		let posZ = Math.floor(-8 + Math.random() * 16);
		if(Math.random() > 0.65){
			e.dimension.spawnEntity("raigen:hailrog_small_hail", {x:e.location.x + posX, y:e.location.y + 16, z:e.location.z + posZ});
		}
		else{
			e.dimension.spawnEntity("raigen:hailrog_big_hail", {x:e.location.x + posX, y:e.location.y + 16, z:e.location.z + posZ});
		}
	},
	"raigen:ovex_aoe": function(e){
		let coords = [
			{ x:2.5, z:0 },
			{ x:-2.5, z:0 },
			{ x:0, z:2.5 },
			{ x:0, z:-2.5 },
			{ x:1.75, z:1.75 },
			{ x:-1.75, z:-1.75 },
			{ x:-1.75, z:1.75 },
			{ x:1.75, z:-1.75 }
		];
		for(var i = 0; i < coords.length; i++){
			e.dimension.createExplosion({x:e.location.x + coords[i].x, y:e.location.y, z:e.location.z + coords[i].z}, 2, {source:e});
		}
		e.triggerEvent("raigen:event_die");
	},
	"raigen:ovex_passive": function(e){
		let rX = -3 + Math.random() * 6;
		let rZ = -3 + Math.random() * 6;
		e.dimension.createExplosion({x:e.location.x + rX, y:e.location.y, z:e.location.z + rZ}, 2, {source:e});
	}
};

function shockAttack(e){
	if(e.target == undefined) return;
	wireParticle(e.location, e.target.location, 0.85, e, "raigen:vesparc_shock");
	e.target?.applyDamage(16);
}