import { MolangVariableMap } from '@minecraft/server';
import { Vector3 } from 'common/math.js';
import { doAbsorb, absorbDome, carrotOffset, yLessOffset } from 'essential/functions.js';

// offset entity radius damage blacklist
export function damageArea(offset, source, radius, damage, blacklist){
	source.dimension.getEntities({
		location:yLessOffset(source, offset),
		maxDistance:radius,
		excludeFamilies:blacklist
	}).forEach(entity =>{
		entity.applyDamage(damage);
	});
}

// generates at a given location
// particle entity location variables_array
export function genParticles(p, e, l, v){
	if(v == undefined){
		try{ e.dimension.spawnParticle(p, l) }
		catch{}
	}
	else{
		var variables = new MolangVariableMap();
		v.forEach(i =>{
			let name = i.split('-')[0];
			let val = parseFloat(i.split('-')[1]);
			variables.setFloat(`variable.${name}`, val);
		})
		e.dimension.spawnParticle(p, l, variables);
	}
}

// creates a wire of particles from
// one coordinate to another.
// from_coordinate to_coordinate step entity particle variables_array
export function wireParticle(f, t, s, e, p, v) {
  let distanceX = Math.abs(t.x - f.x);
  let distanceY = Math.abs(t.y - f.y);
  let distanceZ = Math.abs(t.z - f.z);
	let ssX = Math.ceil(distanceX / s);
	let ssY = Math.ceil(distanceY / s);
	let ssZ = Math.ceil(distanceZ / s);
	let total = Math.max(ssX, ssY, ssZ);
	for (let i = 0; i < total; i++){
		let currentX = t.x > f.x ? f.x + (i / (total - 1)) * distanceX : f.x - (i / (total - 1)) * distanceX;
		let currentY = t.y > f.y ? f.y + (i / (total - 1)) * distanceY : f.y - (i / (total - 1)) * distanceY;
		let currentZ = t.z > f.z ? f.z + (i / (total - 1)) * distanceZ : f.z - (i / (total - 1)) * distanceZ;
		genParticles(p, e, { x:currentX, y:currentY, z:currentZ }, v)
	}
}

// summons a whisper
// source item block block_face
export function summonWhisper(s, i, b, f){
	if(i.typeId != 'minecraft:flint_and_steel') return;
	if(b.typeId != 'minecraft:soul_soil') return;
	if(b.above().typeId != 'minecraft:skeleton_skull') return;
	if(f != 'Up') return;
	let spot = {x:b.x + 0.5, y:b.y + 1, z:b.z + 0.5};
	b.dimension.setBlockType(b.above().location, 'minecraft:air');
	b.dimension.spawnEntity('raigen:whisper<raigen:event_summoned>', spot);
	b.dimension.playSound('mob.blaze.shoot', b.location);
	b.dimension.spawnParticle('raigen:whisper_summon', spot);
}

// mimics the jump mechanic of a vanilla slime
// entity y-power x-multiplier
export function jumpMechanic(entity, yPower, goRandom, sound, xMultiplier){
	if(!entity.isOnGround) return
	xMultiplier == undefined ? xMultiplier = 1 : null;
	if(entity.target != undefined){
		entity.applyImpulse({x:entity.getViewDirection().x * xMultiplier, y:yPower, z:entity.getViewDirection().z * xMultiplier});
		sound == undefined || sound == "" ? null : entity.dimension.playSound(sound, entity.location);
	}
	else{
		if(goRandom){
			let xRand = (-1 + Math.random() * 2);
			let zRand = (-1 + Math.random() * 2);
			entity.applyImpulse({x:xRand * xMultiplier, y:yPower, z:zRand * xMultiplier});
			entity.dimension.playSound(sound, entity.location);
		}
	}
}

// Increases the growth stage of nether warts
// player item block
export function useCharredBoneMeal(player, item, block){
	if(block.typeId != 'minecraft:nether_wart' || item.typeId != 'raigen:charred_bone_meal') return;
	let age = block.permutation.getState('age');
	if(age == 3) return;
	if(player.getGameMode() == 'creative'){
		block.setPermutation(block.permutation.withState('age', 3));
	}
	else{
		block.setPermutation(block.permutation.withState('age', age + 1));
		player.runCommand('clear @s raigen:charred_bone_meal 0 1');
	}
	block.dimension.spawnParticle('raigen:charred_smoke', {x:block.x + 0.5, y:block.y, z:block.z + 0.5});
}

// teleports the victim into the air when hit by a chorupse
// attacker victim
export function chorupseHit(attacker, victim){
	if(attacker.typeId != 'raigen:chorupse') return;
	if(victim == undefined) return;
	victim.teleport({x:victim.location.x, y:victim.location.y + (10 + Math.random() * 5), z:victim.location.z});
}

// drags the victim towards the venomite
// attacker victim
export function venomiteHit(attacker, victim){
	if(attacker.typeId != 'raigen:venomite') return;
	if(victim == undefined) return;
	absorbDome(1, attacker, 2, 3, ["arthropod", "inanimate"]);
}

// launches the victim when a rumbler uppercuts
// attacker victim
export function rumblerHit(attacker, victim){
	if(attacker.typeId != "raigen:rumbler") return;
	if(attacker.getComponent("skin_id").value != 1) return;
	if(victim == undefined) return;
	victim.applyKnockback(0, 0, 0, 0.5 + Math.random());
}

// blinds the victim if the victim is a player
// attacker victim
export function freakHit(attacker, victim){
	if(attacker.typeId != "raigen:freak") return;
	if(victim == undefined) return;
	if(victim.typeId != "minecraft:player") return;
	victim.runCommand("camera @s fade time 0 2 3 color 0 0 0");
}

// allows the user to double jump when
// using a pressure charge
export function usePressureCharge(item, player){
	if(item.typeId != "raigen:pressure_charge") return;
	player.applyKnockback(0, 0, 0, 0.75);
	player.dimension.playSound("random.fizz", player.location);
	player.runCommand("clear @s[m=!c] raigen:pressure_charge 0 1");
	player.runCommand("particle raigen:pressure_ring ~ ~0.1 ~");
}

// sets mobs on fire when being interacted
// with ember powder
export function useEmberPowder(item, player, target){
	if(item.typeId != "raigen:ember_powder") return;
	target.setOnFire(8, true);
	target.dimension.playSound("mob.blaze.shoot", target.location);
	player.runCommand("clear @s[m=!c] raigen:ember_powder 0 1");
}

// sets the embers' victim on fire
export function embersHit(attacker, victim){
	if(attacker.typeId != "raigen:embers") return;
	if(victim == undefined) return;
	victim.setOnFire(8, true);
}

// hurts those that hurt tusoch
export function hitTusoch(attacker, victim){
	if(victim.typeId != "raigen:tusoch") return;
	attacker.applyDamage(1);
}

// pulls the naverin's target when it gets
// hit by its sprite
export function naverinSpriteHit(attacker, victim){
	if(attacker.typeId != "raigen:naverin_sprite") return;
	attacker.dimension.getEntities({
		location:attacker.location,
		type:"raigen:naverin"
	}).forEach(i =>{
		if(i.getDynamicProperty("raigen:entity_id") == attacker.getDynamicProperty("raigen:entity_id")){
			doAbsorb(i, victim, 5);
		}
	});
	attacker.triggerEvent("raigen:event_despawn");
}

// pulls down a sinkrae's victim
// when attacking
export function sinkraeHit(attacker, victim){
	if(attacker.typeId != "raigen:sinkrae") return;
	if(!attacker.isInWater) return;
	attacker.dimension.spawnParticle("raigen:sinkrae_whirlpool", victim.location);
	try{
		victim.applyImpulse({x:0, y:-1.5, z:0});
	}
	catch{
		victim.applyKnockback(0, 0, 0, -1.5);
	}
	attacker.teleport({x:victim.location.x, y:victim.location.y - 1, z:victim.location.z});
}

// pulls down a sinkrae's victim
// when attacking
export function gorgeHit(attacker, victim){
	if(attacker.typeId != "raigen:gorge") return;
	if(attacker.getComponent("skin_id").value != 1) return;
	victim.teleport(attacker.location);
}

// knocks victims back when attack by
// a rockram
export function rockramHit(attacker, victim){
	if(attacker.typeId != "raigen:rockram") return;
	victim.applyKnockback(attacker.getViewDirection().x, attacker.getViewDirection().z, 4.5, 0.5);
}

// either inflicts the status effect
// or spawns a book
export function useCursedTome(item, player){
	if(!item.hasTag("raigen:cursed_tome")) return;
	let tome = item.typeId.split("tome_of_")[1];
	if(Math.random() < 0.85){
		player.runCommand(`effect @e[r=5] ${tome} 10 1`);
		player.dimension.spawnParticle(`raigen:use_tome_${tome}`, player.location);
		player.dimension.spawnParticle(`raigen:cursed_tome_${tome}_ring`, {x:player.location.x + 0.5, y:player.location.y, z:player.location.z + 0.5});
		player.dimension.playSound("mob.elderguardian.curse", player.location);
	}
	else{
		player.runCommand(`clear @s[m=!c] raigen:cursed_tome_of_${tome} 0 1`);
		player.dimension.spawnEntity(`raigen:cursed_tome<raigen:event_${tome}>`, player.location);
	}
}

// ovex fluid explodes when interacted with
export function ovexFluid(block, player, item){
	if(block.typeId != "raigen:ovex_fluid") return;
	if(item.typeId != "minecraft:flint_and_steel" && item.typeId != "minecraft:fire_charge") return;
	if(item.typeId == "minecraft:fire_charge"){
		player.runCommand("clear @s[m=!c] fire_charge 0 1");
	}
	block.dimension.createExplosion(block.location, 2);
	block.dimension.setBlockType(block.location, "minecraft:air");
}